unit Clsgenfm;

interface

uses
	SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
	Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, DBTables, DB;

type
	TMain = class(TForm)
		Panel1: TPanel;
		TableNameEdit: TEdit;
		Label1: TLabel;
		GenerateBtn: TBitBtn;
		BitBtn2: TBitBtn;
		Table: TTable;
		DataSource1: TDataSource;
    BitBtn1: TBitBtn;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    ClassEdit: TEdit;
    BaseClassEdit: TEdit;
    OutputEdit: TEdit;
    OpenDialog: TOpenDialog;
		ClearBtn: TBitBtn;
		procedure GenerateBtnClick(Sender: TObject);
		procedure BitBtn1Click(Sender: TObject);
		procedure ClearBtnClick(Sender: TObject);
	private
		{ Private declarations }
		out: TextFile;
		TableName: string;

		procedure InfoComments;

		procedure WriteSetProcedures(fd: TFieldDef);
		procedure WriteGetProcedures(fd: TFieldDef);

		function GetVarType(fd: TFieldDef): string;
		function GetVarName(fd: TFieldDef): string;
    function GetAsType(fd: TFieldDef): string;
	public
		{ Public declarations }
	end;

var
	Main: TMain;

implementation

{$R *.DFM}

const CUnknown = '***UNKNOWN***';

procedure TMain.GenerateBtnClick(Sender: TObject);
var i: Integer;
	fda: TFieldDefs;
	fd: TFieldDef;
begin
	Table.DatabaseName := ExtractFilePath(TableNameEdit.Text);
	Table.TableName := ExtractFileName(TableNameEdit.Text);
	Table.Active := True;

	TableName := Copy(Table.TableName, 1, Pos('.', Table.TableName) - 1);

	AssignFile(out, OutputEdit.Text);
	ReWrite(out);

	InfoComments;

	WriteLn(out, 'Unit ' +
		Copy(OutputEdit.Text, 1, Pos('.', OutputEdit.Text) - 1) + ';');
	WriteLn(out);
	WriteLn(out, 'interface');
	WriteLn(out);
	WriteLn(out, 'Uses Classes, BusinessObject, DBTables, Global;');
	WriteLn(out);

	{ Class header }
	WriteLn(out, 'type ' + ClassEdit.Text + ' = class(' + BaseClassEdit.Text + ')');
	WriteLn(out, 'protected');

	fda := Table.FieldDefs;

	{ Create private member variable definitions }
	for i := 0 to fda.Count - 1 do
	begin
		fd := fda.Items[i];

		WriteLn(out, ^I + GetVarName(fd) + ': ' + GetVarType(fd) + ';');
	end;
	WriteLn(out);

	WriteLn(out, 'private');
	WriteLn(out, ^I+'procedure CloseDown;');
  WriteLn(out);

	{ Protected functions }
	WriteLn(out, 'protected');
	WriteLn(out, ^I+'{ Protected functions }');
	WriteLn(out, ^I+'procedure ReadyTable;');
	WriteLn(out, ^I+'procedure Initialize;');

	{ Start of public section }
	WriteLn(out);
	WriteLn(out, 'public');

	{ Constructors and destructors }
	WriteLn(out, ^I + '{ Constructor and Destructor }');
	WriteLn(out, ^I + 'constructor Create;');
	WriteLn(out, ^I + 'destructor Free; override;');
	WriteLn(out);

	{ Load and Save }
	WriteLn(out, ^I + '{ Load and Save }');
	WriteLn(out, ^I + 'function Load: Boolean; override;');
	WriteLn(out, ^I + 'function Save: Boolean; override;');
	WriteLn(out, ^I + 'function DBDelete: Boolean; override;');
	WriteLn(out);

	{ Equality and Assignment }
	WriteLn(out, ^I + '{ Equality and Assignment }');
	WriteLn(out, ^I + 'function IsEqual(ob: ' + BaseClassEdit.Text +
		'): Boolean; override;');
	WriteLn(out, ^I + 'procedure Assign(ob: ' + BaseClassEdit.Text + '); override;');
	WriteLn(out);

	{ Get's and Set's section }
	WriteLn(out, ^I + '{ Get''s and Set''s }');

	for i := 0 to fda.Count - 1 do
	begin
		fd := fda.Items[i];

		{ Do the Set's first }
		Write(out, ^I + 'procedure ');
		WriteSetProcedures(fd);

		{ Get functions for member variables }
		Write(out, ^I + 'function ');
		WriteGetProcedures(fd);

		WriteLn(out);
	end;

	{ End of class definition }
	WriteLn(out, 'end;');
	WriteLn(out);

	{ Begin implementation section }
	WriteLn(out, 'implementation');
	WriteLn(out);

	WriteLn(out, '{ Unit specific variables }');
	WriteLn(out, 'var');
	WriteLn(out, ^I + 'Table: TTable;');
	WriteLn(out, ^I + 'TableIsInitialized: Boolean;');
	WriteLn(out, ^I + 'LiveObjectCount: Word;');
	WriteLn(out);
	WriteLn(out, 'const CTableName = ''' + UpperCase(TableName) + ''';');
	WriteLn(out);

	{ Protected functions -- ReadyTable }
	WriteLn(out, 'procedure ' + ClassEdit.Text + '.ReadyTable;');
	WriteLn(out, 'begin');
	WriteLn(out, ^I+'if not TableIsInitialized then');
	WriteLn(out, ^I^I+'Initialize;');
	WriteLn(out);
	WriteLn(out, ^I+'if Table.Active = False then');
	WriteLn(out, ^I^I+'Table.Open;');
	WriteLn(out, 'end;');
	WriteLn(out);

	{ Initialize }
	WriteLn(out, 'procedure ' + ClassEdit.Text + '.Initialize;');
	WriteLn(out, 'begin');
	WriteLn(out, ^I+'Table := TTable.Create(nil);');
	WriteLn(out, ^I+'with Table do');
	WriteLn(out, ^I+'begin');
	WriteLn(out, ^I^I+'DatabaseName := CDatabaseName;');
	WriteLn(out, ^I^I+'TableName := CTableName;');
	WriteLn(out, ^I+'end;');
	WriteLn(out);
	WriteLn(out, ^I+'TableIsInitialized := True;');
	WriteLn(out, 'end;');
	WriteLn(out);

	{ CloseDown function }
	WriteLn(out, 'procedure ' + ClassEdit.Text + '.CloseDown;');
	WriteLn(out, 'begin');
	WriteLn(out, ^I+'if TableIsInitialized then');
	WriteLn(out, ^I+'begin');
	WriteLn(out, ^I^I+'Table.Close;');
	WriteLn(out, ^I^I+'Table.Free;');
	WriteLn(out, ^I^I+'TableIsInitialized := False;');
	WriteLn(out, ^I+'end;');
	WriteLn(out, ^I+'LiveObjectCount := 0;');
	WriteLn(out, 'end;');
	WriteLn(out);

	{ Constructor body }
	WriteLn(out, ^I + '{ Constructor and Destructor }');
	WriteLn(out, 'constructor ' + ClassEdit.Text + '.Create;');
	WriteLn(out, 'begin');
	WriteLn(out, ^I + 'inherited Create;');
	WriteLn(out);
	WriteLn(out, ^I+'Inc(LiveObjectCount);');

	for i := 0 to fda.Count - 1 do
	begin
		fd := fda.Items[i];

		if fd.DataType = ftMemo then
			WriteLn(out, ^I + GetVarName(fd) + ' := ' + GetVarType(fd) + '.Create;');
	end;

	WriteLn(out, 'end;');
	WriteLn(out);

	{ Destructor body }
	WriteLn(out, 'destructor ' + ClassEdit.Text + '.Free;');
	WriteLn(out, 'begin');
	WriteLn(out, ^I+'if LiveObjectCount > 1 then');
	WriteLn(out, ^I^I+'Dec(LiveObjectCount)');
	WriteLn(out, ^I+'else');
	WriteLn(out, ^I^I+'CloseDown;');

	for i := 0 to fda.Count - 1 do
	begin
		fd := fda.Items[i];

		if fd.DataType = ftMemo then
			WriteLn(out, ^I + GetVarName(fd) + '.Free;');
	end;

	WriteLn(out, 'end;');
	WriteLn(out);

	{ Load }
	WriteLn(out, 'function ' + ClassEdit.Text + '.Load: Boolean;');
	WriteLn(out, 'begin');
	WriteLn(out, ^I + 'Result := True;');
	WriteLn(out);
	WriteLn(out, ^I + 'ReadyTable;');
	WriteLn(out);
	WriteLn(out, ^I + 'if Table.FindKey([' + GetVarName(fda.Items[0]) + ']) then');
	WriteLn(out, ^I+'begin');

	{ Start at second field since first field is keyfield }
	for i := 1 to fda.Count - 1 do
	begin
		fd := fda.Items[i];

		if fd.DataType <> ftMemo then
			WriteLn(out, ^I^I + GetVarName(fd) + ' := Table.FindField(''' + fd.Name +
				''').' + GetAsType(fd) + ';')
		else
			WriteLn(out, ^I^I + GetVarName(fd) + '.Assign(Table.FindField(''' + fd.Name +
				'''));');
	end;

	WriteLn(out);
	WriteLn(out, ^I^I+'ClearModified;');
	WriteLn(out, ^I^I+'ClearNew;');

	WriteLn(out, ^I+'end');
	WriteLn(out, ^I+'else');
	WriteLn(out, ^I^I+'Result := False;');
	WriteLn(out, 'end;');
	WriteLn(out);

	{ Save }
	WriteLn(out, 'function ' + ClassEdit.Text + '.Save: Boolean;');
	WriteLn(out, 'begin');
	WriteLn(out, ^I+'Result := True;');
	WriteLn(out);
	WriteLn(out, ^I+'ReadyTable;');
	WriteLn(out);
	WriteLn(out, ^I+'if IsNew then');
	WriteLn(out, ^I+'begin');
	WriteLn(out, ^I^I+'Table.Append;');
	WriteLn(out, ^I^I+GetVarName(fda.Items[0]) + ' := GetNextKey(CTableName);');
	WriteLn(out, ^I+'end');
	WriteLn(out, ^I+'else');
	WriteLn(out, ^I+'begin');
	WriteLn(out, ^I^I+'if Table.FindKey([' + GetVarName(fda.Items[0]) + ']) then');
	WriteLn(out, ^I^I^I+'Table.Edit');
	WriteLn(out, ^I^I+'else');
	WriteLn(out, ^I^I^I+'Result := False;');
	WriteLn(out, ^I+'end;');
	WriteLn(out);
	WriteLn(out, ^I+'with Table do');
	WriteLn(out, ^I+'begin');

	for i := 0 to fda.Count - 1 do
	begin
		fd := fda.Items[i];

		if fd.DataType <> ftMemo then
			WriteLn(out, ^I^I+'FindField(''' + fd.name + ''').' + GetAsType(fd) +
				' := ' + GetVarName(fd) + ';')
		else
			WriteLn(out, ^I^I+'FindField(''' + fd.name + ''').Assign(' +
				GetVarName(fd) + ');');
	end;

	WriteLn(out, ^I+'end;');
	WriteLn(out);
	WriteLn(out, ^I+'Table.Post;');
	WriteLn(out);
	WriteLn(out, ^I+'ClearModified;');
  WriteLn(out, ^I+'ClearNew;');
	WriteLn(out, 'end;');
	WriteLn(out);

	{ Delete }
	WriteLn(out, 'function ' + ClassEdit.Text + '.DBDelete: Boolean;');
	WriteLn(out, 'begin');
	WriteLn(out, ^I + 'Result := True;');
	WriteLn(out);
	WriteLn(out, ^I + 'ReadyTable;');
	WriteLn(out);
	WriteLn(out, ^I + 'if Table.FindKey([' + GetVarName(fda.Items[0]) + ']) then');
	WriteLn(out, ^I^I+'Table.Delete');
	WriteLn(out, ^I+'else');
	WriteLn(out, ^I^I+'Result := False;');
	WriteLn(out, 'end;');
	WriteLn(out);

	{ IsEqual }
	WriteLn(out, 'function ' + ClassEdit.Text + '.IsEqual(ob: ' +
		BaseClassEdit.Text + '): Boolean;');
	WriteLn(out, 'var obj: ' + ClassEdit.Text + ';');
	WriteLn(out, 'begin');
	WriteLn(out, ^I + 'Result := inherited IsEqual(ob);');
	WriteLn(out);
	WriteLn(out, ^I+'obj := ' + ClassEdit.Text + '(ob);');
	WriteLn(out);

	for i := 0 to fda.Count - 1 do
	begin
		fd := fda.Items[i];

		WriteLn(out, ^I + 'if ' + GetVarName(fd) + ' <> obj.Get' + fd.Name + ' then');
		WriteLn(out, ^I^I + 'Result := False;');
		WriteLn(out);
	end;

	WriteLn(out, 'end;');
	WriteLn(out);

	{ Assign }
	WriteLn(out, 'procedure ' + ClassEdit.Text + '.Assign(ob: ' +
		BaseClassEdit.Text + ');');
	WriteLn(out, 'var obj: ' + ClassEdit.Text + ';');
	WriteLn(out, 'begin');
	WriteLn(out, ^I + 'inherited Assign(ob);');
	WriteLn(out);
	WriteLn(out, ^I+'obj := ' + ClassEdit.Text + '(ob);');
	WriteLn(out);

	for i := 0 to fda.Count - 1 do
	begin
		fd := fda.Items[i];

		if fd.DataType = ftMemo then
			WriteLn(out, ^I + GetVarName(fd) + '.Assign(obj.Get' + fd.Name + ');')
		else
			WriteLn(out, ^I + GetVarName(fd) + ' := obj.Get' + fd.Name + ';');
	end;
	WriteLn(out, 'end;');
	WriteLn(out);

	{ Begin body of Get's and Set's functions }
	for i := 0 to fda.Count - 1 do
	begin
		fd := fda.Items[i];

		{ Do the Set's first }
		Write(out, 'procedure ' + ClassEdit.Text + '.');
		WriteSetProcedures(fd);
		WriteLn(out, 'begin');

		if fd.DataType = ftMemo then
			WriteLn(out, ^I + GetVarName(fd) + '.Assign(' + fd.Name + ');')
		else
			WriteLn(out, ^I + GetVarName(fd) + ' := ' + fd.Name + ';');

		WriteLn(out, ^I + 'SetModified;');
		WriteLn(out, 'end;');
		WriteLn(out);

		Write(out, 'function ' + ClassEdit.Text + '.');
		WriteGetProcedures(fd);
		WriteLn(out, 'begin');
		WriteLn(out, ^I + 'Result := ' + GetVarName(fd) + ';');
		WriteLn(out, 'end;');

		WriteLn(out);
	end;

	{ Initialization }
	WriteLn(out, 'Initialization');
	WriteLn(out);
	WriteLn(out, ^I+'LiveObjectCount := 0;');
  WriteLn(out, ^I+'TableIsInitialized := False;');
	WriteLn(out);

	{ End of unit }
	WriteLn(out, 'end.');

	CloseFile(out);
	Table.Active := False;
end;

procedure TMain.WriteSetProcedures(fd: TFieldDef);
begin
	Write(out, 'Set' + fd.Name + '(' + fd.Name + ': ');

	case fd.DataType of
		ftBoolean:
			WriteLn(out, 'Boolean);');

		ftInteger:
			WriteLn(out, 'Longint);');

		ftCurrency:
			WriteLn(out, 'Double);');

		ftMemo:
			WriteLn(out, 'TStringList);');

		ftSmallInt:
			WriteLn(out, 'Integer);');

		ftDate,
		ftDateTime:
			WriteLn(out, 'TDateTime);');

		ftString:
			WriteLn(out, 'string);');
	else
		WriteLn(out, CUnknown + ';');
	end;
end;

procedure TMain.WriteGetProcedures(fd: TFieldDef);
begin
	Write(out, 'Get' + fd.Name + ': ');

	case fd.DataType of
		ftBoolean:
			WriteLn(out, 'Boolean;');

		ftInteger:
			WriteLn(out, 'Longint;');

		ftCurrency:
    	WriteLn(out, 'Double;');

		ftMemo:
			WriteLn(out, 'TStringList;');

		ftSmallInt:
			WriteLn(out, 'Integer;');

		ftDate,
		ftDateTime:
			WriteLn(out, 'TDateTime;');

		ftString:
			WriteLn(out, 'string;');
	else
		WriteLn(out, CUnknown + ';');
	end;
end;

function TMain.GetVarName(fd: TFieldDef): string;
begin
	Result := 'm_';

	case fd.DataType of
		ftDate,	ftMemo, ftDateTime, ftString, ftCurrency:
			Result := Result + fd.Name;

		ftBoolean:
			Result := Result + 'b' + fd.Name;

		ftInteger:
			Result := Result + 'l' + fd.Name;

		ftSmallInt:
			Result := Result + 'i' + fd.Name;
	else
			Result := Result + CUnKnown;
	end;
end;

function TMain.GetVarType(fd: TFieldDef): string;
begin
	case fd.DataType of
		ftMemo: Result := 'TStringList';
		ftDate, ftDateTime: Result := 'TDateTime';
		ftString: Result := 'string[' + IntToStr(fd.Size) + ']';
		ftBoolean: Result := 'Boolean';
		ftInteger: Result := 'Longint';
    ftCurrency: Result := 'Double';
		ftSmallInt: Result := 'Integer';
	else
		Result := CUnKnown;
	end;
end;

function TMain.GetAsType(fd: TFieldDef): string;
begin
	case fd.DataType of
		ftDate, ftDateTime, ftCurrency: Result := 'AsFloat';
		ftString: Result := 'AsString';
		ftBoolean: Result := 'AsBoolean';
		ftInteger, ftSmallInt: Result := 'AsInteger';
	else
		Result := CUnKnown;
	end;
end;


procedure TMain.BitBtn1Click(Sender: TObject);
begin
	if OpenDialog.Execute = True then
		TableNameEdit.Text := OpenDialog.FileName;  	
end;

procedure TMain.ClearBtnClick(Sender: TObject);
begin
	ClassEdit.Text := '';
	BaseClassEdit.Text := '';
	TableNameEdit.Text := '';
	OutputEdit.Text := '';
end;

procedure TMain.InfoComments;
begin
	WriteLn(out, '{ This code generated by ClassGen v1.1 }');
	WriteLn(out);
end;

end.
